% varargout = half_space_gf_tol(testing, sources, k0, oGd, varargin)
% If an argument is present after sPol it is componentString, and only that component
% is returned. Otherwise the tangental components are returned.
% the fourth argument is always a parameter, in this case oGd.

function [field, nPoints] = adaptive_ndsdp(EvalAt, Source, permittivityLowerRegion,...
    permittivityUpperRegion, componentString, Options)
k0 = 2*pi/Source.lambda;
isMagneticSource = ~(lower(Source.polarizationString(2)) == 'm');
nStart = 8;
ratio = 2;
hasMetTol = false(size(EvalAt.x));

Options.nPoints = nStart;
Options.useAdaptive = false;
field = zeros(size(EvalAt.x));
fieldNext = field;
nPoints = field;
fieldCurr = ndsdp(EvalAt, Source, permittivityLowerRegion, ...
    permittivityUpperRegion, componentString, Options);
% calc field in homogeneous space as an estimate for field amplitude
averagePermittivity = (permittivityLowerRegion+permittivityUpperRegion)/2;
if lower(componentString(2)) == 'z'
    estimate = LS_space(EvalAt.x, EvalAt.y, Source.x, Source.y, k0, ...
        averagePermittivity, componentString, isMagneticSource);
else
    yComponentString = [componentString(1), 'x'];
    xComponentString = [componentString(1), 'y'];
    xEstimate = LS_space(EvalAt.x, EvalAt.y, Source.x, Source.y, k0, ...
        averagePermittivity, xComponentString, isMagneticSource);
    yEstimate = LS_space(EvalAt.x, EvalAt.y, Source.x, Source.y, k0, ...
        averagePermittivity, yComponentString, isMagneticSource);
    estimate = sqrt(abs(xEstimate).^2+abs(yEstimate).^2);
end

while any(~hasMetTol(:))
    EvalAtNotMetTol.x = EvalAt.x(~hasMetTol);
    EvalAtNotMetTol.y = EvalAt.y(~hasMetTol);
    Options.nPoints = round(Options.nPoints*ratio);
    fieldNext(~hasMetTol) = ndsdp(EvalAtNotMetTol, Source, permittivityLowerRegion, ...
        permittivityUpperRegion, componentString, Options);
    hasMetTol(~hasMetTol) = hasMetTol(~hasMetTol) |...
        (abs(fieldCurr(~hasMetTol) - fieldNext(~hasMetTol)) <= Options.tol*abs(estimate(~hasMetTol)));
    if (round(Options.nPoints*ratio) > Options.maxPoints) && any(~hasMetTol(:))
        warning('NDSDP:maxPointsLimit', 'Integration point limit reached: Options.nPoints > Options.maxPoints')
        disp('Problem encountered for observation point coordinates:')
        disp('EvalAt.x = ')
        disp(EvalAt.x(~hasMetTol))
        disp('EvalAt.y = ')
        disp(EvalAt.y(~hasMetTol))
        disp('and source at coordinates:')
        disp('Source.x = ')
        disp(Source.x)
        disp('Source.y = ')
        disp(Source.y)
        hasMetTol(~hasMetTol) = true;
    end
    if any(hasMetTol(:))
        field(hasMetTol) = fieldNext(hasMetTol);
    end
    if any(~hasMetTol(:))
        fieldCurr(~hasMetTol) = fieldNext(~hasMetTol);
    end
    nPoints(hasMetTol & (nPoints==0)) = Options.nPoints;
end
